# Copyright 2018 ACSONE SA/NV (<http://acsone.eu>)
# License LGPL-3.0 or later (http://www.gnu.org/licenses/lgpl.html).

import ast
import os

from dodoo import odoo

MANIFEST_NAMES = ("__manifest__.py", "__openerp__.py")


class NoManifestFound(Exception):
    pass


class ModuleNotFound(Exception):
    pass


def get_manifest_path(addon_dir):
    for manifest_name in MANIFEST_NAMES:
        manifest_path = os.path.join(addon_dir, manifest_name)
        if os.path.isfile(manifest_path):
            return manifest_path


def parse_manifest(s):
    return ast.literal_eval(s)


def read_manifest(addon_dir):
    manifest_path = get_manifest_path(addon_dir)
    if not manifest_path:
        raise NoManifestFound("no Odoo manifest found in %s" % addon_dir)
    with open(manifest_path) as mf:
        return parse_manifest(mf.read())


def find_addons(addons_dir, installable_only=True):
    """ yield (addon_name, addon_dir, manifest) """
    for addon_name in sorted(os.listdir(addons_dir)):
        addon_dir = os.path.join(addons_dir, addon_name)
        try:
            manifest = read_manifest(addon_dir)
        except NoManifestFound:
            continue
        if installable_only and not manifest.get("installable", True):
            continue
        yield addon_name, addon_dir, manifest


def expand_dependencies(module_names, include_auto_install=False, include_active=False):
    """ Return a set of module names with their transitive
    dependencies.  This method does not need an Odoo database,
    but requires the addons path to be initialized.
    """

    def add_deps(name):
        if name in res:
            return
        res.add(name)
        path = odoo.modules.get_module_path(name)
        if not path:
            raise ModuleNotFound(name)
        manifest = read_manifest(path)
        for dep in manifest.get("depends", ["base"]):
            add_deps(dep)

    res = set()
    for module_name in module_names:
        add_deps(module_name)
    if include_active:
        for module_name in sorted(odoo.modules.module.get_modules()):
            module_path = odoo.modules.get_module_path(module_name)
            manifest = read_manifest(module_path)
            if manifest.get("active"):
                add_deps(module_name)
    if include_auto_install:
        auto_install_list = []
        for module_name in sorted(odoo.modules.module.get_modules()):
            module_path = odoo.modules.get_module_path(module_name)
            manifest = read_manifest(module_path)
            if manifest.get("auto_install"):
                auto_install_list.append((module_name, manifest))
        retry = True
        while retry:
            retry = False
            for module_name, manifest in auto_install_list:
                if module_name in res:
                    continue
                depends = set(manifest.get("depends", ["base"]))
                if depends.issubset(res):
                    # all dependencies of auto_install module are
                    # installed so we add it
                    add_deps(module_name)
                    # retry, in case an auto_install module depends
                    # on other auto_install modules
                    retry = True
    return res



from __future__ import absolute_import, division, print_function, unicode_literals

import os
import os.path as path
import subprocess
import textwrap

import pytest
from dodoo import odoo, odoo_bin
from future import standard_library

# Copyright 2018 ACSONE SA/NV (<http://acsone.eu>)
# License LGPL-3.0 or later (http://www.gnu.org/licenses/lgpl.html).


standard_library.install_aliases()


# This hack is necessary because the way CliRunner patches
# stdout is not compatible with the Odoo logging initialization
# mechanism. Logging is therefore tested with subprocesses.
odoo.netsvc._logger_init = True

DB_PREFIX = path.basename(path.abspath(path.join(__file__, "../.."))).replace(".", "_")


def _init_odoo_db(dbname, test_addons_dir=None):
    subprocess.check_call(["createdb", dbname])
    cmd = [odoo_bin, "-d", dbname, "-i", "base", "--stop-after-init"]
    if test_addons_dir:
        addons_path = [
            os.path.join(odoo.__path__[0], "addons"),
            os.path.join(odoo.__path__[0], "..", "addons"),
            test_addons_dir,
        ]
        cmd.append("--addons-path")
        cmd.append(",".join(addons_path))
    subprocess.check_call(cmd)


def _drop_db(dbname):
    subprocess.check_call(["dropdb", dbname])


@pytest.fixture(scope="module")
def odoodb(request):
    dbname = (DB_PREFIX + "-test-{}").format(odoo.release.version_info[0])
    test_addons_dir = getattr(request.module, "test_addons_dir", "")
    _init_odoo_db(dbname, test_addons_dir)
    try:
        yield dbname
    finally:
        _drop_db(dbname)


@pytest.fixture(scope="function")
def odoocfg(request, tmpdir):
    addons_path = [
        os.path.join(odoo.__path__[0], "addons"),
        os.path.join(odoo.__path__[0], "..", "addons"),
    ]
    test_addons_dir = getattr(request.module, "test_addons_dir", "")
    if test_addons_dir:
        addons_path.append(test_addons_dir)
    odoo_cfg = tmpdir / "odoo.cfg"
    odoo_cfg.write(
        textwrap.dedent(
            """\
        [options]
        addons_path = {}
    """.format(
                ",".join(addons_path)
            )
        )
    )
    yield odoo_cfg



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is part of the dodoo-tester (R) project.
# Copyright (c) 2018 XOE Corp. SAS
# Authors: David Arnold, et al.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function, unicode_literals

import os
from builtins import super

import click
from dodoo import CommandWithOdooEnv, odoo

from .git import Git


def _get_changed_modules_from_git(git_dir, base_ref="origin/master"):
    """ Returns odoo changed odoo modules as per git diff-index.
    :param base_ref: branch or remote/branch or sha to compare
    :return: List of unique modules changed """

    changed_paths = Git(git_dir).get_changed_paths(base_ref)
    res = []
    for path in changed_paths:
        module_root = odoo.modules.module.get_module_root(path)
        if not module_root:
            continue
        res.append(os.path.basename(module_root))
    return set(res)


class CommandWithOdooEnvExtended(CommandWithOdooEnv):
    def _parse_env_args(self, ctx):
        def _get(flag):
            return ctx.params.get(flag)

        database = _get("database")
        if not database:
            click.BadParameter(
                "The --database parameter is required for testing. You cannot "
                "specify the database in the config file."
            )
        odoo_args = super(CommandWithOdooEnvExtended, self)._parse_env_args(ctx)

        # fmt: off
        # Always present params
        git_dir, ref = _get("git_dir")       # noqa
        include      = _get("include")       # noqa
        exclude      = _get("exclude")       # noqa
        tags         = _get("tags")          # noqa
        # fmt: on

        if tags:
            odoo_args.extend(["--test-tags", tags])
        odoo_args.extend(["--log-db", database])
        odoo_args.extend(["--log-db-level", "warning"])

        changed = _get_changed_modules_from_git(git_dir, ref) if git_dir else set()
        modules = (changed | set(include)) - set(exclude)

        if not modules:
            click.echo("No module to test. Exiting...")
            click.get_current_context().exit()
        odoo_args.extend(["-i", ",".join(modules)])
        odoo_args.extend(["-u", ",".join(modules)])
        odoo_args.extend(["--test-enable"])
        return odoo_args



# Copyright 2018 ACSONE SA/NV (<http://acsone.eu>)
# License LGPL-3.0 or later (http://www.gnu.org/licenses/lgpl.html).

import os
import subprocess

import pytest

from utils.gitutils import commit_if_needed


@pytest.fixture
def gitdir(tmpdir):
    subprocess.check_call(["git", "init"], cwd=str(tmpdir))
    subprocess.check_call(["git", "config", "user.name", "tester"], cwd=str(tmpdir))
    subprocess.check_call(
        ["git", "config", "user.email", "tester@example.com"], cwd=str(tmpdir)
    )
    yield tmpdir


def _git_ls_files(cwd):
    output = subprocess.check_output(
        ["git", "ls-files"], cwd=str(cwd), universal_newlines=True
    )
    return output.strip().split("\n")


def _git_staged_files(cwd):
    output = subprocess.check_output(
        ["git", "diff", "--cached", "--name-only"],
        cwd=str(cwd),
        universal_newlines=True,
    )
    return output.strip().split("\n")


def _git_add(paths, cwd):
    cmd = ["git", "add", "--"] + paths
    subprocess.check_call(cmd, cwd=str(cwd))


def test_git_commit_if_needed(gitdir):
    assert "file1" not in _git_ls_files(gitdir)
    file1 = gitdir / "file1"
    file1.ensure(file=True)
    assert commit_if_needed([str(file1)], "msg", cwd=str(gitdir))
    assert "file1" in _git_ls_files(gitdir)
    # no change, commit_if_needed returns False
    assert not commit_if_needed([str(file1)], "msg", cwd=str(gitdir))
    # some change
    file1.write("stuff")
    assert commit_if_needed([str(file1)], "msg", cwd=str(gitdir))
    # some unrelated file not in git
    file2 = gitdir / "file2"
    file2.ensure(file=True)
    assert not commit_if_needed([str(file1)], "msg", cwd=str(gitdir))
    assert "file2" not in _git_ls_files(gitdir)
    # some unrelated file in git index
    _git_add([str(file2)], gitdir)
    assert not commit_if_needed([str(file1)], "msg", cwd=str(gitdir))
    assert "file1" not in _git_staged_files(gitdir)
    assert "file2" in _git_staged_files(gitdir)
    # add subdirectory
    dir1 = gitdir / "dir1"
    dir1.ensure(dir=True)
    file3 = dir1 / "file3"
    file3.ensure(file=True)
    assert commit_if_needed([str(file3)], "msg", cwd=str(gitdir))
    assert "dir1/file3" in _git_ls_files(gitdir)


def test_commit_git_ignored(gitdir):
    file1 = gitdir / "file1.pot"
    file1.ensure(file=True)
    gitignore = gitdir / ".gitignore"
    gitignore.write("*.pot\n")
    assert commit_if_needed([str(file1)], "msg", cwd=str(gitdir))
    assert "file1.pot" in _git_ls_files(gitdir)


def test_commit_reldir(gitdir):
    with gitdir.as_cwd():
        os.mkdir("subdir")
        file1 = "subdir/file1"
        with open(file1, "w"):
            pass
        assert commit_if_needed([file1], "msg", cwd="subdir")



# Copyright 2018 ACSONE SA/NV (<http://acsone.eu>)
# License LGPL-3.0 or later (http://www.gnu.org/licenses/lgpl.html).

import os
import subprocess


def commit_if_needed(paths, message, cwd="."):
    paths = [os.path.realpath(p) for p in paths]
    cmd = ["git", "add", "-f", "--"] + paths
    subprocess.check_call(cmd, cwd=cwd)
    cmd = ["git", "diff", "--quiet", "--exit-code", "--cached", "--"] + paths
    r = subprocess.call(cmd, cwd=cwd)
    if r != 0:
        cmd = ["git", "commit", "-m", message, "--"] + paths
        subprocess.check_call(cmd, cwd=cwd)
        return True
    else:
        return False



# Copyright 2018 ACSONE SA/NV (<http://acsone.eu>)
# License LGPL-3.0 or later (http://www.gnu.org/licenses/lgpl.html).

import os

import pytest
from dodoo import odoo

from utils import manifest

HERE = os.path.dirname(__file__)
ADDONS_DIR = os.path.join(HERE, "data/test_manifest")


def test_manifest_find_addons():
    addons = list(manifest.find_addons(ADDONS_DIR))
    assert len(addons) == 1
    assert addons[0][0] == "addon1"


def test_manifest_find_addons_uninstallable():
    addons = list(manifest.find_addons(ADDONS_DIR, installable_only=False))
    assert len(addons) == 2
    assert addons[0][0] == "addon1"
    assert addons[1][0] == "addon_uninstallable"


def test_manifest_expand_dependencies():
    res = manifest.expand_dependencies(["auth_signup", "base_import"])
    assert "auth_signup" in res
    assert "mail" in res  # dependency of auth_signup
    assert "base_import" in res
    assert "base" in res  # obviously
    assert "web" in res  # base_import depends on web
    if odoo.release.version_info[0] < 12:
        assert "auth_crypt" not in res
    else:
        assert "iap" not in res  # iap is auto_install


def test_manifest_expand_dependencies_auto_install():
    res = manifest.expand_dependencies(["auth_signup"], include_auto_install=True)
    assert "auth_signup" in res
    assert "base" in res  # obviously
    if odoo.release.version_info[0] < 12:
        assert "auth_crypt" in res  # auth_crypt is autoinstall
    else:
        assert "iap" in res  # iap is auto_install
    assert "web" in res  # web is autoinstall
    assert "base_import" in res  # base_import is indirect autoinstall


def test_manifest_expand_dependencies_not_found():
    with pytest.raises(manifest.ModuleNotFound):
        manifest.expand_dependencies(["not_a_module"])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is part of the dodoo-tester (R) project.
# Copyright (c) 2018 XOE Corp. SAS
# Authors: David Arnold, et al.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function, unicode_literals

import logging
from builtins import str

import click
import dodoo
import pytest as pytest_orig
from future import standard_library

from .commands import CommandWithOdooEnvExtended
from .env import OdooPyTestExecution, OdooTestExecution
from .pytest import OdooPlugin

standard_library.install_aliases()


# from utils import manifest, gitutils

_logger = logging.getLogger(__name__)


@click.command(
    cls=CommandWithOdooEnvExtended,
    env_options={
        "database_must_exist": False,
        "environment_manager": OdooTestExecution,
    },
)
@dodoo.options.addons_path_opt(True)
@dodoo.options.db_opt(True)
@click.option(
    "--git-dir",
    default=(None, None),
    nargs=2,
    type=(click.Path(), str),
    help="Autodetect changed modules (through git).",
)
@click.option("--include", "-i", multiple=True, help="Force test run on those modules.")
@click.option(
    "--exclude",
    "-e",
    multiple=True,
    help="Force excluding those modules from tests. Even if "
    "a change has been detected.",
)
@click.option("--tags", "-t", multiple=True, help="Filter on those test tags.")
def test(env, git_dir, include, exclude, tags):
    """ Run Odoo tests through unittest.
    """
    pass


@click.command(
    cls=dodoo.commands.CommandWithOdooEnv,
    env_options={
        "database_must_exist": True,
        "environment_manager": OdooPyTestExecution,
    },
)
@dodoo.options.addons_path_opt(True)
@dodoo.options.db_opt(True)
@click.argument("args", nargs=-1, required=True)
def pytest(env, args):
    """ Run Odoo tests through pytest.
    """
    pytest_orig.main(list(args), plugins=[OdooPlugin()])


if __name__ == "__main__":  # pragma: no cover
    test()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is part of the dodoo-tester (R) project.
# Copyright (c) 2018 XOE Corp. SAS
# Authors: David Arnold, et al.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, see <http://www.gnu.org/licenses/>.
#

# import mock
from __future__ import absolute_import, division, print_function, unicode_literals

import os

from click.testing import CliRunner
from dodoo import odoo
from future import standard_library

from dodoo_tester.cli import pytest, test

standard_library.install_aliases()


# from dodoo import odoo


# from ..utils import manifest, gitutils


def test_execute_crm_tests(odoodb, odoocfg):
    result = CliRunner().invoke(
        test,
        [
            "--config",
            str(odoocfg),
            "--database",
            str(odoodb),
            "--include",
            "crm",
            "--include",
            "project",
            "--exclude",
            "project",
        ],
    )
    assert result.exit_code == 0


def test_execute_pytest(odoodb, odoocfg):

    testfile_path = os.path.join(
        odoo.__path__[0], "addons", "base", "tests", "test_expression.py"
    )
    result = CliRunner().invoke(
        pytest, ["--config", str(odoocfg), "--database", str(odoodb), testfile_path]
    )
    assert result.exit_code == 0



from __future__ import absolute_import, division, print_function, unicode_literals

import logging
import re
import textwrap
from builtins import range

from future import standard_library

standard_library.install_aliases()

_logger = logging.getLogger(__name__)
# Always output summary to standard err;
# disregard odoo logging settings.
_logger.handlers = []
_logger.addHandler(logging.StreamHandler())

BLACK, RED, GREEN, YELLOW, BLUE, MAGENTA, CYAN, WHITE, _NOTHING, DEFAULT = range(10)
# The background is set with 40 plus the number of the color, and the foreground with 30
# These are the sequences need to get colored ouput
RESET_SEQ = "\033[0m"
COLOR_SEQ = "\033[1;%dm"
BOLD_SEQ = "\033[1m"
COLOR_PATTERN = "{}{}%s{}".format(COLOR_SEQ, COLOR_SEQ, RESET_SEQ)


def process(logs):
    warnings = []
    errors = []
    critical = []
    failed = 0

    incident_fmt = """



    {_name}
    ---------------------------------------
    {_path}:{_line}
    {_func}
    _______________________________________

{message}
"""
    file_regex = r'File ".*?odoo'

    def _process_msg(msg):
        try:
            # PY3
            return textwrap.indent(re.sub(file_regex, 'File "odoo', msg), "    *   ")
        except AttributeError:
            wrapper = textwrap.TextWrapper(
                initial_indent="    *   ", subsequent_indent="    *   "
            )
            return wrapper.wrap(re.sub(file_regex, 'File "odoo', msg))

    consolidated_logs = {}
    for (_name, level, message, _path, _func, _line) in logs:
        key = (_name, level, _path, _func, _line)
        if key not in consolidated_logs:
            consolidated_logs[key] = message
        else:
            consolidated_logs[key] = consolidated_logs[key] + "\n" + message

    for (_name, level, _path, _func, _line), message in consolidated_logs.items():
        if "FAILED" in message:
            failed += 1
        if level == "WARNING":
            message = _process_msg(message)
            warnings.append(incident_fmt.format(**locals()))
        if level == "ERROR":
            message = _process_msg(message)
            errors.append(incident_fmt.format(**locals()))
        if level == "CRITICAL":
            message = _process_msg(message)
            critical.append(incident_fmt.format(**locals()))

    # Summary Format
    summary_fmt = COLOR_PATTERN % (
        30 + CYAN,
        40 + DEFAULT,
        """
    SUMMARY
    ======================================================
    WARNING: {: 3}   ERROR: {: 3}   CRITICAL: {: 3}  FAILED: {: 3}
    ------------------------------------------------------
""".format(
            len(warnings), len(errors), len(critical), failed
        ),
    )

    # Warnings Format
    warnings_fmt = (
        COLOR_PATTERN
        % (
            30 + YELLOW,
            40 + DEFAULT,
            """
    Warnings:
    ======================================================
    {}
""".format(
                ("").join(warnings)
            ),
        )
        if len(warnings)
        else ""
    )

    # Errors Format
    errors_fmt = (
        COLOR_PATTERN
        % (
            30 + RED,
            40 + DEFAULT,
            """
    Errors:
    ======================================================
    {}
""".format(
                ("").join(errors)
            ),
        )
        if len(errors)
        else ""
    )

    # Critical Format
    critical_fmt = (
        COLOR_PATTERN
        % (
            30 + RED,
            40 + YELLOW,
            """
    Critical:
    ======================================================
    {}
""".format(
                ("").join(critical)
            ),
        )
        if len(critical)
        else ""
    )

    # Final Output Format
    _logger.info(
        """


    %s


    %s


    %s


    %s
""",
        summary_fmt,
        warnings_fmt,
        errors_fmt,
        critical_fmt,
    )

    return not bool(len(errors) or len(critical) or failed)



# -*- coding: utf-8 -*-
# Copyright 2019 CXOE Corp. SAS
# Copyright 2016 Camptocamp SA
# Copyright 2015 Odoo
# License AGPL-3.0 or later (http://www.gnu.org/licenses/lgpl.html)

import os
import sys

import _pytest
import _pytest.python
import py.code
import py.error
import py.path
from dodoo import odoo

# import pytest
# import threading


try:
    import openerp  # noqa

    odoo_namespace = "openerp"
except ImportError:  # Odoo >= 10.0
    import odoo  # noqa

    odoo_namespace = "odoo"


# @pytest.fixture(scope="session", autouse=True)
# def load_registry():
#     # Initialize the registry before running tests.
#     # If we don't do that, the modules will be loaded *inside* of the first
#     # test we run, which would trigger the launch of the postinstall tests
#     # (because we force 'test_enable' to True and the at end of the loading of
#     # the registry, the postinstall tests are run when test_enable is enabled).
#     # And also give wrong timing indications.
#     # Finally we enable `testing` flag on current thread
#     # since Odoo sets it when loading test suites.
#     threading.currentThread().testing = True
#     odoo.registry(odoo.tests.common.get_db_name())


# @pytest.fixture(scope="module", autouse=True)
# def enable_odoo_test_flag():
#     # When we run tests through Odoo, test_enable is always activated, and some
#     # code might rely on this (for instance to selectively disable database
#     # commits). When we run the tests through pytest, the flag is not
#     # activated, and if it was activated globally, it would make odoo start all
#     # tests in addition to the tests we are running through pytest.  If we
#     # enable the option only in the scope of the tests modules, we won't
#     # interfere with the odoo's loading of modules, thus we are good.
#     odoo.tools.config["test_enable"] = True
#     yield
#     odoo.tools.config["test_enable"] = False


# Original code of xmo-odoo:
# https://github.com/odoo-dev/odoo/commit/95a131b7f4eebc6e2c623f936283153d62f9e70f
class OdooTestModule(_pytest.python.Module):
    """ Should only be invoked for paths inside Odoo addons
    """

    def _importtestmodule(self):
        # copy/paste/modified from original: removed sys.path injection &
        # added Odoo module prefixing so import within modules is correct
        try:
            pypkgpath = self.fspath.pypkgpath()
            pkgroot = pypkgpath.dirpath()
            sep = self.fspath.sep
            names = self.fspath.new(ext="").relto(pkgroot).split(sep)
            if names[-1] == "__init__":
                names.pop()
            modname = ".".join(names)
            # modules installed with pip for odoo<=9 are into the
            # odoo_addons namespace. Makes module part of the odoo namespace
            if modname.startswith("odoo_addons"):
                modname = (
                    odoo_namespace + ".addons." + modname.replace("odoo_addons.", "")
                )
            # for modules in openerp/addons, since there is a __init__ the
            # module name is already fully qualified (maybe?)
            if (
                not modname.startswith(odoo_namespace + ".addons.")
                and modname != odoo_namespace + ".addons"
                and modname != odoo_namespace
            ):
                modname = odoo_namespace + ".addons." + modname

            __import__(modname)
            mod = sys.modules[modname]
            if self.fspath.basename == "__init__.py":
                # we don't check anything as we might
                # we in a namespace package ... too icky to check
                return mod
            modfile = mod.__file__
            if modfile[-4:] in (".pyc", ".pyo"):
                modfile = modfile[:-1]
            elif modfile.endswith("$py.class"):
                modfile = modfile[:-9] + ".py"
            if modfile.endswith(os.path.sep + "__init__.py"):
                if self.fspath.basename != "__init__.py":
                    modfile = modfile[:-12]
            try:
                issame = self.fspath.samefile(modfile)
            except py.error.ENOENT:
                issame = False
            if not issame:
                raise self.fspath.ImportMismatchError(modname, modfile, self)
        except SyntaxError:
            raise self.CollectError(py.code.ExceptionInfo().getrepr(style="short"))
        except self.fspath.ImportMismatchError:
            e = sys.exc_info()[1]
            raise self.CollectError(
                "import file mismatch:\n"
                "imported module %r has this __file__ attribute:\n"
                "  %s\n"
                "which is not the same as the test file we want to collect:\n"
                "  %s\n"
                "HINT: remove __pycache__ / .pyc files and/or use a "
                "unique basename for your test file modules" % e.args
            )
        self.config.pluginmanager.consider_module(mod)
        return mod

    def __repr__(self):
        return "<Module {!r}>".format(getattr(self, "name", None))


class OdooTestPackage(_pytest.python.Package, OdooTestModule):
    """Package with odoo module lookup.

    Any python module inside the package will be imported with
    the prefix `odoo.addons`.

    This class is used to prevent loading odoo modules in duplicate,
    which happens if a module is loaded with and without the prefix.
    """

    def __repr__(self):
        return "<Package {!r}>".format(getattr(self, "name", None))


class OdooPlugin(object):
    @staticmethod
    def pytest_pycollect_makemodule(path, parent):
        if path.basename == "__init__.py":
            return OdooTestPackage(path, parent)
        else:
            return OdooTestModule(path, parent)



# Copyright 2018 ACSONE SA/NV (<http://acsone.eu>)
# License LGPL-3.0 or later (http://www.gnu.org/licenses/lgpl.html).

import os

from setuptools import find_packages, setup

here = os.path.abspath(os.path.dirname(__file__))

setup(
    name="dodoo-tester",
    description="dodoo tester script",
    long_description="\n".join(
        (
            open(os.path.join(here, "README.rst")).read(),
            open(os.path.join(here, "CHANGES.rst")).read(),
        )
    ),
    use_scm_version=True,
    packages=find_packages(),
    setup_requires=["setuptools-scm"],
    install_requires=[
        "dodoo>=2.0.0.rc6",
        "future",
        "pyyaml==3.12 ; python_version < '3.7'",
        "pyyaml==3.13 ; python_version >= '3.7'",
    ],
    license="LGPLv3+",
    author="XOE Labs",
    author_email="info@xoe.solutions",
    url="http://github.com/xoe-labs/dodoo-tester",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: "
        "GNU Lesser General Public License v3 or later (LGPLv3+)",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Framework :: Odoo",
    ],
    entry_points="""
        [core_package.cli_plugins]
        test=dodoo_tester.cli:test
        pytest=dodoo_tester.cli:pytest
    """,
)



from contextlib import closing, contextmanager

import click
import psycopg2
from dodoo import odoo

from .format import process


@contextmanager
def OdooTestExecution(self):
    connection = {"dbname": self.database}
    if odoo.tools.config["db_host"]:
        connection.update({"host": odoo.tools.config["db_host"]})
    if odoo.tools.config["db_port"]:
        connection.update({"port": odoo.tools.config["db_port"]})
    if odoo.tools.config["db_user"]:
        connection.update({"user": odoo.tools.config["db_user"]})
    if odoo.tools.config["db_password"]:
        connection.update({"password": odoo.tools.config["db_password"]})
    with closing(psycopg2.connect(**connection)) as conn, conn.cursor() as cr:
        cr.execute(
            """ALTER TABLE ir_logging ADD COLUMN IF NOT EXISTS "test_run" INTEGER;"""
        )
        conn.commit()
    try:
        yield odoo.service.server.start(preload=[self.database], stop=True)
    finally:
        with closing(psycopg2.connect(**connection)) as conn, conn.cursor() as cr:
            cr.execute(
                """
                SELECT name,
                       LEVEL,
                       message,
                       path,
                       func,
                       line
                FROM ir_logging WHERE test_run IS NULL;
                """
            )
            logs = cr.fetchall()
            # Mark logs for subsequent runs to filter.
            cr.execute(
                """
                SELECT DISTINCT test_run
                FROM ir_logging
                WHERE test_run IS NOT NULL;

                """
            )
            next_run = max([run for (run,) in cr.fetchall()] or [0]) + 1
            cr.execute(
                """UPDATE ir_logging SET test_run = %s WHERE test_run IS NULL;""",
                (next_run,),
            )
            conn.commit()
            # PostgreSQL 9.2 renamed pg_stat_activity.procpid to pid:
            # http://www.postgresql.org/docs/9.2/static/release-9-2.html#AEN110389
            pid_col = "pid" if conn.server_version >= 90200 else "procpid"
            cr.execute(
                """
                SELECT pg_terminate_backend(%(pid_col)s)
                FROM pg_stat_activity
                WHERE datname = %%s
                AND %(pid_col)s != pg_backend_pid()"""
                % {"pid_col": pid_col},
                (self.database,),
            )

    ctx = click.get_current_context()
    success = process(logs)
    if not success:
        ctx.fail("Tests failed")


@contextmanager
def OdooPyTestExecution(self):
    odoo.service.server.start(preload=[self.database], stop=True)
    with odoo.api.Environment.manage():
        yield
    connection = {"dbname": self.database}
    if odoo.tools.config["db_host"]:
        connection.update({"host": odoo.tools.config["db_host"]})
    if odoo.tools.config["db_port"]:
        connection.update({"port": odoo.tools.config["db_port"]})
    if odoo.tools.config["db_user"]:
        connection.update({"user": odoo.tools.config["db_user"]})
    if odoo.tools.config["db_password"]:
        connection.update({"password": odoo.tools.config["db_password"]})
    with closing(psycopg2.connect(**connection)) as conn, conn.cursor() as cr:
        # PostgreSQL 9.2 renamed pg_stat_activity.procpid to pid:
        # http://www.postgresql.org/docs/9.2/static/release-9-2.html#AEN110389
        pid_col = "pid" if conn.server_version >= 90200 else "procpid"
        cr.execute(
            """
            SELECT pg_terminate_backend(%(pid_col)s)
            FROM pg_stat_activity
            WHERE datname = %%s
            AND %(pid_col)s != pg_backend_pid()"""
            % {"pid_col": pid_col},
            (self.database,),
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is part of the dodoo-tester (R) project.
# Copyright (c) 2018 XOE Corp. SAS
# Authors: David Arnold, et al.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function, unicode_literals

import subprocess
from builtins import str


class Git(object):
    def __init__(self, git_dir):
        self.git_dir = git_dir

    def run(self, command):
        """Execute git command in bash
        :param list command: Git cmd to execute in self.git_dir
        :return: String output of command executed.
        """
        cmd = ["git", "--git-dir=" + self.git_dir] + command
        try:
            res = subprocess.check_output(cmd)
        except subprocess.CalledProcessError:
            res = None
        # Python 2.7
        if isinstance(res, str):
            res = res.strip("\n")
        # Python 3.X
        if isinstance(res, bytes):
            res = res.strip(b"\n")
        return res

    def fetch_remote(self, ref):
        """ Fetches a remote ref, independent of current refspec config """
        if "/" not in ref:
            return
        parts = ref.split("/", 1)
        remote = parts[0]
        ref_branch = parts[1]
        refmap_option = "--refmap=+refs/heads/{ref_branch}:"
        "refs/remotes/{remote}/{ref_branch}".format(**locals())
        self.run(["fetch", refmap_option] + parts)

    def get_changed_paths(self, base_ref="HEAD"):
        """Get name of items changed in self.git_dir
        This is a wrapper method of git command:
            git diff-index --name-only --cached {base_ref}
        :param base_ref: String of branch or sha base.
            e.g. "master" or "SHA_NUMBER"
        :return: List of name of items changed
        """
        self.fetch_remote(base_ref)
        command = ["diff-index", "--name-only", "--cached", base_ref]
        res = self.run(command)
        items = res.decode("UTF-8").split("\n") if res else []
        return items

    def get_branch_name(self):
        """Get branch name
        :return: String with name of current branch name"""
        command = ["rev-parse", "--abbrev-ref", "HEAD"]
        res = self.run(command)
        return res
